"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const template_1 = __importDefault(require("@babel/template"));
const helper_plugin_utils_1 = require("@babel/helper-plugin-utils");
const FUNCS_PATH = "cua.functions"; // TODO: configurable relative path
const CLIENT_PKG = "@cua.run/client";
let importedFuncs = [];
let clientImported = false;
let program;
exports.default = (0, helper_plugin_utils_1.declare)((api) => {
    return {
        name: "cua.run",
        visitor: {
            ImportDeclaration: (path) => {
                if (path.node.importKind !== "value") {
                    return;
                }
                if (path.node.source.value === CLIENT_PKG) {
                    clientImported = true;
                }
                else if (path.node.source.value.match(FUNCS_PATH)) {
                    importedFuncs.push(...path.node.specifiers
                        .map((sp) => sp.type === "ImportDefaultSpecifier" &&
                        (sp.local.name || sp.local))
                        .filter((v) => !!v));
                    path.remove();
                }
            },
            Program(path) {
                program = path;
                clientImported = false;
                importedFuncs = [];
            },
            CallExpression(path) {
                if (path.node.callee.type !== "Identifier" ||
                    !importedFuncs.includes(path.node.callee.name)) {
                    return;
                }
                if (!clientImported) {
                    const importDeclaration = (0, template_1.default)(`
            import %%importName%% from %%importPath%%
          `);
                    program.unshiftContainer("body", importDeclaration({
                        importName: api.types.identifier("cua"),
                        importPath: api.types.stringLiteral("@cua.run/client"),
                    }));
                    clientImported = true;
                }
                const pushDeclaration = template_1.default.statement(`await cua.push(%%fn%%)`);
                path.replaceWith(pushDeclaration({
                    fn: api.types.stringLiteral(path.node.callee.name),
                }));
            },
        },
    };
});
